@@ -23,7 +23,10 @@
 #'   * A __data frame__ containing both the predictors and the outcome.
 #'
 #' @inheritParams brulee_mlp
-#'
+#' @param optimizer The method used in the optimization procedure. Possible choices
+#'   are `"SGD"`,  `"ADAMw"`, `"Adadelta"`, `"Adagrad"`, `"RMSprop"`, and
+#'   `"LBFGS"`. `"LBFGS"` is the only second-order method, does not use
+#'   batches, and is the default.
 #' @details
 #'
 #' This function fits a linear combination of coefficients and predictors to
@@ -69,7 +72,7 @@
 #'  * `parameters`: A list of some tuning parameter values.
 #'  * `blueprint`: The `hardhat` blueprint data.
 #'
-#' @examples
+#' @examplesIf !brulee:::is_cran_check()
 #' \donttest{
 #' if (torch::torch_is_installed()  & rlang::is_installed(c("recipes", "yardstick", "modeldata"))) {
 #'
@@ -100,29 +103,28 @@
 #'  ames_rec <-
 #'   recipe(Sale_Price ~ Bldg_Type + Neighborhood + Year_Built + Gr_Liv_Area +
 #'          Full_Bath + Year_Sold + Lot_Area + Central_Air + Longitude + Latitude,
-#'          data = ames_train) %>%
+#'          data = ames_train) |>
 #'     # Transform some highly skewed predictors
-#'     step_BoxCox(Lot_Area, Gr_Liv_Area) %>%
+#'     step_BoxCox(Lot_Area, Gr_Liv_Area) |>
 #'     # Lump some rarely occurring categories into "other"
-#'     step_other(Neighborhood, threshold = 0.05)  %>%
+#'     step_other(Neighborhood, threshold = 0.05)  |>
 #'     # Encode categorical predictors as binary.
-#'     step_dummy(all_nominal_predictors(), one_hot = TRUE) %>%
+#'     step_dummy(all_nominal_predictors(), one_hot = TRUE) |>
 #'     # Add an interaction effect:
-#'     step_interact(~ starts_with("Central_Air"):Year_Built) %>%
-#'     step_zv(all_predictors()) %>%
+#'     step_interact(~ starts_with("Central_Air"):Year_Built) |>
+#'     step_zv(all_predictors()) |>
 #'     step_normalize(all_numeric_predictors())
 #'
 #'  set.seed(2)
-#'  fit <- brulee_linear_reg(ames_rec, data = ames_train,
-#'                            epochs = 5, batch_size = 32)
+#'  fit <- brulee_linear_reg(ames_rec, data = ames_train, epochs = 5)
 #'  fit
 #'
 #'  autoplot(fit)
 #'
 #'  library(ggplot2)
 #'
-#'  predict(fit, ames_test) %>%
-#'    bind_cols(ames_test) %>%
+#'  predict(fit, ames_test) |>
+#'    bind_cols(ames_test) |>
 #'    ggplot(aes(x = .pred, y = Sale_Price)) +
 #'    geom_abline(col = "green") +
 #'    geom_point(alpha = .3) +
@@ -130,8 +132,8 @@
 #'    coord_fixed(ratio = 1)
 #'
 #'  library(yardstick)
-#'  predict(fit, ames_test) %>%
-#'    bind_cols(ames_test) %>%
+#'  predict(fit, ames_test) |>
+#'    bind_cols(ames_test) |>
 #'    rmse(Sale_Price, .pred)
 #'
 #'  }
@@ -145,7 +147,12 @@ brulee_linear_reg <- function(x, ...) {
 #' @export
 #' @rdname brulee_linear_reg
 brulee_linear_reg.default <- function(x, ...) {
-  stop("`brulee_linear_reg()` is not defined for a '", class(x)[1], "'.", call. = FALSE)
+  stop(
+    "`brulee_linear_reg()` is not defined for a '",
+    class(x)[1],
+    "'.",
+    call. = FALSE
+  )
 }
 
 # XY method - data frame
@@ -153,19 +160,21 @@ brulee_linear_reg.default <- function(x, ...) {
 #' @export
 #' @rdname brulee_linear_reg
 brulee_linear_reg.data.frame <-
-  function(x,
-           y,
-           epochs = 20L,
-           penalty = 0.001,
-           mixture = 0,
-           validation = 0.1,
-           optimizer = "LBFGS",
-           learn_rate = 1.0,
-           momentum = 0.0,
-           batch_size = NULL,
-           stop_iter = 5,
-           verbose = FALSE,
-           ...) {
+  function(
+    x,
+    y,
+    epochs = 20L,
+    penalty = 0.001,
+    mixture = 0,
+    validation = 0.1,
+    optimizer = "LBFGS",
+    learn_rate = 1.0,
+    momentum = 0.0,
+    batch_size = NULL,
+    stop_iter = 5,
+    verbose = FALSE,
+    ...
+  ) {
     processed <- hardhat::mold(x, y)
 
     brulee_linear_reg_bridge(
@@ -188,19 +197,21 @@ brulee_linear_reg.data.frame <-
 
 #' @export
 #' @rdname brulee_linear_reg
-brulee_linear_reg.matrix <- function(x,
-                                     y,
-                                     epochs = 20L,
-                                     penalty = 0.001,
-                                     mixture = 0,
-                                     validation = 0.1,
-                                     optimizer = "LBFGS",
-                                     learn_rate = 1,
-                                     momentum = 0.0,
-                                     batch_size = NULL,
-                                     stop_iter = 5,
-                                     verbose = FALSE,
-                                     ...) {
+brulee_linear_reg.matrix <- function(
+  x,
+  y,
+  epochs = 20L,
+  penalty = 0.001,
+  mixture = 0,
+  validation = 0.1,
+  optimizer = "LBFGS",
+  learn_rate = 1,
+  momentum = 0.0,
+  batch_size = NULL,
+  stop_iter = 5,
+  verbose = FALSE,
+  ...
+) {
   processed <- hardhat::mold(x, y)
 
   brulee_linear_reg_bridge(
@@ -224,19 +235,21 @@ brulee_linear_reg.matrix <- function(x,
 #' @export
 #' @rdname brulee_linear_reg
 brulee_linear_reg.formula <-
-  function(formula,
-           data,
-           epochs = 20L,
-           penalty = 0.001,
-           mixture = 0,
-           validation = 0.1,
-           optimizer = "LBFGS",
-           learn_rate = 1,
-           momentum = 0.0,
-           batch_size = NULL,
-           stop_iter = 5,
-           verbose = FALSE,
-           ...) {
+  function(
+    formula,
+    data,
+    epochs = 20L,
+    penalty = 0.001,
+    mixture = 0,
+    validation = 0.1,
+    optimizer = "LBFGS",
+    learn_rate = 1,
+    momentum = 0.0,
+    batch_size = NULL,
+    stop_iter = 5,
+    verbose = FALSE,
+    ...
+  ) {
     processed <- hardhat::mold(formula, data)
 
     brulee_linear_reg_bridge(
@@ -260,19 +273,21 @@ brulee_linear_reg.formula <-
 #' @export
 #' @rdname brulee_linear_reg
 brulee_linear_reg.recipe <-
-  function(x,
-           data,
-           epochs = 20L,
-           penalty = 0.001,
-           mixture = 0,
-           validation = 0.1,
-           optimizer = "LBFGS",
-           learn_rate = 1,
-           momentum = 0.0,
-           batch_size = NULL,
-           stop_iter = 5,
-           verbose = FALSE,
-           ...) {
+  function(
+    x,
+    data,
+    epochs = 20L,
+    penalty = 0.001,
+    mixture = 0,
+    validation = 0.1,
+    optimizer = "LBFGS",
+    learn_rate = 1,
+    momentum = 0.0,
+    batch_size = NULL,
+    stop_iter = 5,
+    verbose = FALSE,
+    ...
+  ) {
     processed <- hardhat::mold(x, data)
 
     brulee_linear_reg_bridge(
@@ -294,11 +309,25 @@ brulee_linear_reg.recipe <-
 # ------------------------------------------------------------------------------
 # Bridge
 
-brulee_linear_reg_bridge <- function(processed, epochs, optimizer,
-                                     learn_rate, momentum, penalty, mixture, dropout,
-                                     validation, batch_size, stop_iter, verbose, ...) {
-  if(!torch::torch_is_installed()) {
-    cli::cli_abort("The torch backend has not been installed; use `torch::install_torch()`.")
+brulee_linear_reg_bridge <- function(
+  processed,
+  epochs,
+  optimizer,
+  learn_rate,
+  momentum,
+  penalty,
+  mixture,
+  dropout,
+  validation,
+  batch_size,
+  stop_iter,
+  verbose,
+  ...
+) {
+  if (!torch::torch_is_installed()) {
+    cli::cli_abort(
+      "The torch backend has not been installed; use `torch::install_torch()`."
+    )
   }
 
   f_nm <- "brulee_linear_reg"
@@ -316,7 +345,14 @@ brulee_linear_reg_bridge <- function(processed, epochs, optimizer,
   }
   check_double(penalty, single = TRUE, 0, incl = c(TRUE, TRUE), fn = f_nm)
   check_double(mixture, single = TRUE, 0, 1, incl = c(TRUE, TRUE), fn = f_nm)
-  check_double(validation, single = TRUE, 0, 1, incl = c(TRUE, FALSE), fn = f_nm)
+  check_double(
+    validation,
+    single = TRUE,
+    0,
+    1,
+    incl = c(TRUE, FALSE),
+    fn = f_nm
+  )
   check_double(momentum, single = TRUE, 0, 1, incl = c(TRUE, TRUE), fn = f_nm)
   check_double(learn_rate, single = TRUE, 0, incl = c(FALSE, TRUE), fn = f_nm)
   check_logical(verbose, single = TRUE, fn = f_nm)
@@ -371,8 +407,16 @@ brulee_linear_reg_bridge <- function(processed, epochs, optimizer,
   )
 }
 
-new_brulee_linear_reg <- function( model_obj, estimates, best_epoch, loss,
-                                   dims, y_stats, parameters, blueprint) {
+new_brulee_linear_reg <- function(
+  model_obj,
+  estimates,
+  best_epoch,
+  loss,
+  dims,
+  y_stats,
+  parameters,
+  blueprint
+) {
   if (!inherits(model_obj, "raw")) {
     cli::cli_abort("'model_obj' should be a raw vector.")
   }
@@ -391,34 +435,38 @@ new_brulee_linear_reg <- function( model_obj, estimates, best_epoch, loss,
   if (!inherits(blueprint, "hardhat_blueprint")) {
     cli::cli_abort("'blueprint' should be a hardhat blueprint")
   }
-  hardhat::new_model(model_obj = model_obj,
-                     estimates = estimates,
-                     best_epoch = best_epoch,
-                     loss = loss,
-                     dims = dims,
-                     y_stats = y_stats,
-                     parameters = parameters,
-                     blueprint = blueprint,
-                     class = "brulee_linear_reg")
+  hardhat::new_model(
+    model_obj = model_obj,
+    estimates = estimates,
+    best_epoch = best_epoch,
+    loss = loss,
+    dims = dims,
+    y_stats = y_stats,
+    parameters = parameters,
+    blueprint = blueprint,
+    class = "brulee_linear_reg"
+  )
 }
 
 ## -----------------------------------------------------------------------------
 # Fit code
 
 linear_reg_fit_imp <-
-  function(x, y,
-           epochs = 20L,
-           batch_size = 32,
-           penalty = 0.001,
-           mixture = 0,
-           validation = 0.1,
-           optimizer = "LBFGS",
-           learn_rate = 1,
-           momentum = 0.0,
-           stop_iter = 5,
-           verbose = FALSE,
-           ...) {
-
+  function(
+    x,
+    y,
+    epochs = 20L,
+    batch_size = NULL,
+    penalty = 0.001,
+    mixture = 0,
+    validation = 0.1,
+    optimizer = "LBFGS",
+    learn_rate = 1,
+    momentum = 0.0,
+    stop_iter = 5,
+    verbose = FALSE,
+    ...
+  ) {
     torch::torch_manual_seed(sample.int(10^5, 1)) # TODO doesn't give reproducible results
 
     ## ---------------------------------------------------------------------------
@@ -435,14 +483,14 @@ linear_reg_fit_imp <-
 
     y_dim <- 1
     loss_fn <- function(input, target) {
-      nnf_mse_loss(input, target$view(c(-1,1)))
+      nnf_mse_loss(input, target$view(c(-1, 1)))
     }
 
     if (validation > 0) {
       in_val <- sample(seq_along(y), floor(n * validation))
-      x_val <- x[in_val,, drop = FALSE]
+      x_val <- x[in_val, , drop = FALSE]
       y_val <- y[in_val]
-      x <- x[-in_val,, drop = FALSE]
+      x <- x[-in_val, , drop = FALSE]
       y <- y[-in_val]
     }
 
@@ -456,8 +504,8 @@ linear_reg_fit_imp <-
     loss_label <- "\tLoss (scaled):"
 
     if (optimizer == "LBFGS" & !is.null(batch_size)) {
-     cli::cli_warn("'batch_size' is only used for the SGD optimizer.")
-     batch_size <- NULL
+      cli::cli_warn("'batch_size' is only used for the SGD optimizer.")
+      batch_size <- NULL
     }
     if (is.null(batch_size)) {
       batch_size <- nrow(x)
@@ -465,9 +513,13 @@ linear_reg_fit_imp <-
       batch_size <- min(batch_size, nrow(x))
     }
 
-
     ## ---------------------------------------------------------------------------
     # Convert to index sampler and data loader
+
+    or_dtype <- torch::torch_get_default_dtype()
+    on.exit(torch::torch_set_default_dtype(or_dtype))
+    torch::torch_set_default_dtype(torch::torch_float64())
+
     ds <- matrix_to_dataset(x, y)
     dl <- torch::dataloader(ds, batch_size = batch_size)
 
@@ -479,7 +531,7 @@ linear_reg_fit_imp <-
     ## ---------------------------------------------------------------------------
     # Initialize model and optimizer
     model <- linear_reg_module(ncol(x))
-    loss_fn <- make_penalized_loss(loss_fn, model, penalty, mixture)
+    loss_fn <- make_penalized_loss(loss_fn, model, penalty, mixture, optimizer)
     optimizer_obj <- set_optimizer(optimizer, model, learn_rate, momentum)
 
     ## ---------------------------------------------------------------------------
@@ -499,6 +551,12 @@ linear_reg_fit_imp <-
 
     # Optimize parameters
     for (epoch in 1:epochs) {
+      learn_rate <- set_learn_rate(epoch - 1, learn_rate, type = "none", ...)
+
+      for (i in seq_along(optimizer_obj$param_groups)) {
+        optimizer_obj$param_groups[[i]]$lr <- learn_rate
+      }
+
       # training loop
       coro::loop(
         for (batch in dl) {
@@ -527,7 +585,9 @@ linear_reg_fit_imp <-
       loss_vec[epoch] <- loss_curr
 
       if (is.nan(loss_curr)) {
-        cli::cli_warn("Current loss in NaN. Training wil be stopped.")
+        cli::cli_warn(
+          "Early stopping occurred at epoch {epoch} due to numerical overflow of the loss function."
+        )
         break()
       }
 
@@ -548,16 +608,14 @@ linear_reg_fit_imp <-
         lapply(model$state_dict(), function(x) torch::as_array(x$cpu()))
 
       if (verbose) {
-        msg <- paste("epoch:", epoch_chr[epoch], loss_label,
-                     signif(loss_curr, 3), loss_note)
-
-        cli::cli_inform(msg)
+        cli::cli_inform(
+          "epoch: {epoch_chr[epoch]}, learn rate: {signif(learn_rate, 3)}, {loss_label} {signif(loss_curr, 3)}"
+        )
       }
 
       if (poor_epoch == stop_iter) {
         break()
       }
-
     }
 
     ## ---------------------------------------------------------------------------
@@ -569,10 +627,14 @@ linear_reg_fit_imp <-
       best_epoch = best_epoch,
       dims = list(p = p, n = n, h = 0, y = y_dim, features = colnames(x)),
       y_stats = y_stats,
-      parameters = list(learn_rate = learn_rate,
-                        penalty = penalty, mixture = mixture,
-                        validation = validation,
-                        batch_size = batch_size, momentum = momentum)
+      parameters = list(
+        learn_rate = learn_rate,
+        penalty = penalty,
+        mixture = mixture,
+        validation = validation,
+        batch_size = batch_size,
+        momentum = momentum
+      )
     )
   }
 
@@ -584,7 +646,7 @@ linear_reg_module <-
       self$fc1 <- torch::nn_linear(num_pred, 1)
     },
     forward = function(x) {
-      x %>% self$fc1()
+      x |> self$fc1()
     }
   )
 
@@ -595,4 +657,3 @@ print.brulee_linear_reg <- function(x, ...) {
   cat("Linear regression\n\n")
   brulee_print(x)
 }
-
