@@ -15,7 +15,7 @@ test_that("basic binomial mlp LBFGS", {
  bin_te <- modeldata::sim_classification(1000)
 
  rec <-
-  recipe(class ~ ., data = bin_tr) %>%
+  recipe(class ~ ., data = bin_tr) |>
   step_normalize(all_predictors())
  num_class <- length(levels(bin_tr$class))
 
@@ -46,9 +46,9 @@ test_that("basic binomial mlp LBFGS", {
 
  expect_error(
   bin_pred_lbfgs <-
-   predict(bin_fit_lbfgs, bin_te) %>%
-   bind_cols(predict(bin_fit_lbfgs, bin_te, type = "prob")) %>%
-   bind_cols(bin_te) %>%
+   predict(bin_fit_lbfgs, bin_te) |>
+   bind_cols(predict(bin_fit_lbfgs, bin_te, type = "prob")) |>
+   bind_cols(bin_te) |>
    select(starts_with(".pred"), class),
   regex = NA)
 
@@ -68,7 +68,7 @@ test_that("basic binomial mlp LBFGS", {
 
  # Did it learn anything?
  bin_brier_lbfgs <-
-  bin_pred_lbfgs %>%
+  bin_pred_lbfgs |>
   yardstick::brier_class(class, .pred_class_1)
 
  expect_true(bin_brier_lbfgs$.estimate < (1 - 1/num_class)^2)
@@ -91,7 +91,7 @@ test_that("basic binomial mlp SGD", {
  bin_te <- modeldata::sim_classification(1000)
 
  rec <-
-  recipe(class ~ ., data = bin_tr) %>%
+  recipe(class ~ ., data = bin_tr) |>
   step_normalize(all_predictors())
  num_class <- length(levels(bin_tr$class))
 
@@ -107,7 +107,7 @@ test_that("basic binomial mlp SGD", {
               dropout = .1,
               hidden_units = 5,
               optimize = "SGD",
-              batch_size = 64,
+              batch_size = 64L,
               momentum = 0.5,
               learn_rate = 0.1)},
   regex = NA)
@@ -123,16 +123,16 @@ test_that("basic binomial mlp SGD", {
               dropout = .1,
               hidden_units = 5,
               optimize = "SGD",
-              batch_size = 64,
+              batch_size = 64L,
               momentum = 0.5,
               learn_rate = 0.1)},
   regex = NA)
 
  expect_error(
   bin_pred_sgd <-
-   predict(bin_fit_sgd, bin_te) %>%
-   bind_cols(predict(bin_fit_sgd, bin_te, type = "prob")) %>%
-   bind_cols(bin_te) %>%
+   predict(bin_fit_sgd, bin_te) |>
+   bind_cols(predict(bin_fit_sgd, bin_te, type = "prob")) |>
+   bind_cols(bin_te) |>
    select(starts_with(".pred"), class),
   regex = NA)
 
@@ -152,7 +152,7 @@ test_that("basic binomial mlp SGD", {
 
  # Did it learn anything?
  bin_brier_sgd <-
-  bin_pred_sgd %>%
+  bin_pred_sgd |>
   yardstick::brier_class(class, .pred_class_1)
 
  expect_true(bin_brier_sgd$.estimate < (1 - 1/num_class)^2)
@@ -175,49 +175,49 @@ test_that("binomial mlp case weights", {
  bin_te <- modeldata::sim_classification(1000, intercept = 1)
 
  rec <-
-  recipe(class ~ ., data = bin_tr) %>%
+  recipe(class ~ ., data = bin_tr) |>
   step_normalize(all_predictors())
  num_class <- length(levels(bin_tr$class))
 
  # ------------------------------------------------------------------------------
 
  expect_error({
-  set.seed(392)
+  set.seed(391)
   weighted <-
    brulee_mlp(rec,
               bin_tr,
-              epochs = 200,
+              epochs = 5,
               hidden_units = 5,
               rate_schedule = "cyclic",
               class_weights = 10,
-              learn_rate = 0.1)},
+              learn_rate = 0.01)},
   regex = NA)
 
  expect_error(
   weighted_pred <-
-   predict(weighted, bin_te) %>%
-   bind_cols(predict(weighted, bin_te, type = "prob")) %>%
-   bind_cols(bin_te) %>%
+   predict(weighted, bin_te) |>
+   bind_cols(predict(weighted, bin_te, type = "prob")) |>
+   bind_cols(bin_te) |>
    select(starts_with(".pred"), class),
   regex = NA)
 
 
  expect_error({
-  set.seed(392)
+  set.seed(391)
   unweighted <-
    brulee_mlp(rec,
               bin_tr,
-              epochs = 200,
+              epochs = 5,
               hidden_units = 5,
               rate_schedule = "cyclic",
-              learn_rate = 0.1)},
+              learn_rate = 0.01)},
   regex = NA)
 
  expect_error(
   unweighted_pred <-
-   predict(unweighted, bin_te) %>%
-   bind_cols(predict(unweighted, bin_te, type = "prob")) %>%
-   bind_cols(bin_te) %>%
+   predict(unweighted, bin_te) |>
+   bind_cols(predict(unweighted, bin_te, type = "prob")) |>
+   bind_cols(bin_te) |>
    select(starts_with(".pred"), class),
   regex = NA)
 
